// Copyright © 2022-2025 Gerald B. Cox
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

package environ

import (
	"log/slog" // Added for logging warnings/errors during token parsing
	"os"
	"strings"
	"sync" // New: Import for sync.Once
)

// userTokens stores the mapping of (lowercase) username to ListenBrainz API token.
// It's populated once from the USER_TOKENS environment variable.
var userTokens = make(map[string]string)
var once sync.Once // Ensures populateUserTokens is called only once

// populateUserTokens initializes the userTokens map from the USER_TOKENS environment variable.
// This function is designed to be concurrency-safe and idempotent.
func populateUserTokens() {
	once.Do(func() {
		tokenString := os.Getenv("USER_TOKENS")
		if tokenString == "" {
			slog.Error("USER_TOKENS environment variable is not set. API token retrieval will fail.")
			return
		}

		pairs := strings.Split(tokenString, ",")
		for _, pair := range pairs {
			parts := strings.SplitN(pair, ":", 2) // Split at most once
			if len(parts) == 2 {
				// Store user names in lowercase for case-insensitive matching later
				userTokens[strings.ToLower(parts[0])] = parts[1]
			} else {
				slog.Warn("Malformed USER_TOKENS entry, skipping", "entry", pair)
			}
		}
		slog.Debug("Environ: User tokens loaded", "count", len(userTokens))
	})
}

// GetApiToken retrieves the ListenBrainz API token for a given username.
// It ensures that the USER_TOKENS environment variable is parsed exactly once.
// Usernames are case-insensitive.
func GetApiToken(userName string) string {
	populateUserTokens() // Ensure tokens are loaded (only happens once)
	token, ok := userTokens[strings.ToLower(userName)]
	if !ok {
		slog.Warn("Environ: No ListenBrainz token found for user", "user", userName)
		return ""
	}
	return token
}
