// In your project, create a new directory: beetbrainz/jellyfin/
// And save this content as beetbrainz/jellyfin/jellyfin.go

// Copyright © 2022-2025 Gerald B. Cox
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

package jellyfin

import (
	"bytes"
	"encoding/json"
	"errors"
	"fmt"
	"io"
	"log/slog"
	"net/http"
	"strings"

	"codeberg.org/gbcox/beetbrainz/common"
	"codeberg.org/gbcox/beetbrainz/types"
)

// JellyfinWebhookRequest represents the relevant parts of the Jellyfin webhook payload.
// This structure now directly matches the flat JSON sent by Jellyfin.
type JellyfinWebhookRequest struct {
	ServerId         string `json:"ServerId"`
	ServerName       string `json:"ServerName"`
	ServerVersion    string `json:"ServerVersion"`
	ServerUrl        string `json:"ServerUrl"`
	NotificationType string `json:"NotificationType"` // e.g., "PlaybackStart", "PlaybackStop", "PlaybackProgress"
	Timestamp        string `json:"Timestamp"`
	UtcTimestamp     string `json:"UtcTimestamp"`
	Name             string `json:"Name"` // Track Title
	Overview         string `json:"Overview"`
	Tagline          string `json:"Tagline"`
	ItemId           string `json:"ItemId"`
	ItemType         string `json:"ItemType"` // e.g., "Audio"
	RunTimeTicks     int64  `json:"RunTimeTicks"`
	RunTime          string `json:"RunTime"`
	Year             int    `json:"Year"`
	PremiereDate     string `json:"PremiereDate"`
	Album            string `json:"Album"`  // Album Name
	Artist           string `json:"Artist"` // Primary Artist Name

	// Provider fields for MusicBrainz
	Provider_musicbrainzartist       string `json:"Provider_musicbrainzartist"`
	Provider_musicbrainzalbumartist  string `json:"Provider_musicbrainzalbumartist"`
	Provider_musicbrainzalbum        string `json:"Provider_musicbrainzalbum"`
	Provider_musicbrainzreleasegroup string `json:"Provider_musicbrainzreleasegroup"`
	Provider_musicbrainztrack        string `json:"Provider_musicbrainztrack"`

	// Audio Stream fields (Audio_0_...)
	Audio_0_Title      string `json:"Audio_0_Title"`
	Audio_0_Type       string `json:"Audio_0_Type"`
	Audio_0_Language   string `json:"Audio_0_Language"`
	Audio_0_Codec      string `json:"Audio_0_Codec"`
	Audio_0_Channels   int    `json:"Audio_0_Channels"`
	Audio_0_Bitrate    int    `json:"Audio_0_Bitrate"`
	Audio_0_SampleRate int    `json:"Audio_0_SampleRate"`
	Audio_0_Default    bool   `json:"Audio_0_Default"`

	// Playback related fields
	PlaybackPositionTicks int64  `json:"PlaybackPositionTicks"`
	PlaybackPosition      string `json:"PlaybackPosition"`
	MediaSourceId         string `json:"MediaSourceId"`
	IsPaused              bool   `json:"IsPaused"`
	IsAutomated           bool   `json:"IsAutomated"`
	DeviceId              string `json:"DeviceId"`
	DeviceName            string `json:"DeviceName"`
	ClientName            string `json:"ClientName"`
	PlayMethod            string `json:"PlayMethod"`
	PlayedToCompletion    bool   `json:"PlayedToCompletion"`
	Id                    string `json:"Id"` // This appears to be the Session ID or similar unique ID
	UserId                string `json:"UserId"`
	NotificationUsername  string `json:"NotificationUsername"` // User's name for notification
	Client                string `json:"Client"`
	LastActivityDate      string `json:"LastActivityDate"`
	LastPlaybackCheckIn   string `json:"LastPlaybackCheckIn"`
	RemoteEndPoint        string `json:"RemoteEndPoint"`
}

// HandleRequest is the entry point for Jellyfin webhooks.
func HandleRequest(w http.ResponseWriter, r *http.Request) {
	if r.Method != "POST" {
		slog.Warn("Jellyfin webhook received unsupported method", "method", r.Method, "expected_method", "POST")
		http.Error(w, "Method Not Allowed", http.StatusMethodNotAllowed)
		return
	}

	rq, err := parseRequest(r)
	if err != nil {
		slog.Error("Failed to parse Jellyfin webhook request", "error", err)
		http.Error(w, fmt.Sprintf("Bad Request: %s", err.Error()), http.StatusBadRequest)
		return
	}

	// Pass the parsed request to the common handler for processing (scrobbling logic)
	common.HandleRequest(rq)

	w.WriteHeader(http.StatusOK)
}

// parseRequest parses the incoming HTTP request into a common.Request structure.
func parseRequest(r *http.Request) (*common.Request, error) {
	var jellyfinRequest JellyfinWebhookRequest
	var err error

	// Read the raw request body first.
	bodyBytes, err := io.ReadAll(r.Body)
	if err != nil {
		slog.Error("Failed to read Jellyfin request body", "error", err)
		return nil, fmt.Errorf("failed to read request body: %w", err)
	}

	// Re-set the request body for the JSON decoder to read from.
	r.Body = io.NopCloser(bytes.NewBuffer(bodyBytes))

	// Attempt to decode the JSON.
	err = json.NewDecoder(r.Body).Decode(&jellyfinRequest)
	if err != nil {
		// Log a snippet of the body with the JSON decode error for more context
		bodySnippet := string(bodyBytes)
		if len(bodySnippet) > 500 { // Limit snippet length to avoid overly long logs
			bodySnippet = bodySnippet[:500] + "..."
		}
		slog.Error("Failed to decode JSON from Jellyfin request body", "error", err, "raw_body_snippet", bodySnippet)
		return nil, fmt.Errorf("failed to decode JSON from Jellyfin request body: %w", err)
	}

	// --- Basic Validation and Event Handling ---
	if jellyfinRequest.NotificationType == "" {
		return nil, errors.New("Jellyfin webhook payload missing 'NotificationType' field")
	}
	// Only process 'Audio' items for scrobbling
	if jellyfinRequest.ItemType != "Audio" {
		return nil, fmt.Errorf("Jellyfin webhook 'ItemType' is '%s', only 'Audio' (track) is supported for scrobbling", jellyfinRequest.ItemType)
	}
	if jellyfinRequest.NotificationUsername == "" {
		return nil, errors.New("Jellyfin webhook payload missing 'NotificationUsername'")
	}
	// Ensure essential music metadata is present
	if jellyfinRequest.Name == "" || jellyfinRequest.Album == "" || jellyfinRequest.Artist == "" {
		return nil, errors.New("Jellyfin webhook payload missing required music metadata (Name, Album, or Artist)")
	}

	// Map Jellyfin event types to common event types for Beetbrainz
	var commonEvent string

	switch jellyfinRequest.NotificationType {
	case "PlaybackStart":
		commonEvent = common.EventPlaybackStart
	case "PlaybackStop":
		commonEvent = common.EventPlaybackStop
	case "PlaybackProgress":
		// Beetbrainz does not typically scrobble on progress events to avoid excessive calls.
		return nil, fmt.Errorf("Jellyfin webhook 'NotificationType' is 'PlaybackProgress', currently unsupported for scrobbling")
	default:
		return nil, fmt.Errorf("unsupported Jellyfin webhook 'NotificationType': %s", jellyfinRequest.NotificationType)
	}

	rq := &common.Request{
		Event:     commonEvent,
		MediaType: strings.ToLower(jellyfinRequest.ItemType), // Normalize to "audio"
		User:      jellyfinRequest.NotificationUsername,      // Use the string directly for User
		Item: &types.MediaItem{ // Use types.MediaItem
			Artist: jellyfinRequest.Artist,
			Album:  jellyfinRequest.Album,
			Track:  jellyfinRequest.Name,
			Source: "jellyfin", // ✅ Tower ID goes here
		},
	}

	return rq, nil
}
