// Copyright © 2022-2025 Gerald B. Cox
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

package mpris

import (
	"bytes"
	"encoding/json"
	"errors"
	"fmt"
	"io"
	"log/slog"
	"net/http"
	"strings"

	"codeberg.org/gbcox/beetbrainz/common"
	"codeberg.org/gbcox/beetbrainz/types"
)

type ExternalUrl struct {
	Name string `json:"Name"`
	Url  string `json:"Url"`
}

type Item struct {
	Type         string            `json:"Type"`
	Track        string            `json:"Name"`
	Album        string            `json:"Album"`
	AlbumArtist  string            `json:"AlbumArtist"`
	Artists      []string          `json:"Artists"`
	ExternalUrls []ExternalUrl     `json:"ExternalUrls"`
	ProviderIds  map[string]string `json:"ProviderIds"`
}

func (item *Item) String() string {
	artist := item.AlbumArtist
	if len(item.Artists) > 0 && item.Artists[0] != "" {
		artist = strings.Join(item.Artists, ", ")
	}
	return fmt.Sprintf("%s - %s (%s)", artist, item.Track, item.Album)
}

func (item *Item) AsMediaItem() *types.MediaItem {
	mediaItem := &types.MediaItem{
		Track: item.Track,
		Album: item.Album,
	}

	if len(item.Artists) > 0 && item.Artists[0] != "" {
		mediaItem.Artist = strings.Join(item.Artists, ", ")
	} else {
		mediaItem.Artist = item.AlbumArtist
	}

	if item.ProviderIds != nil {
		if id, ok := item.ProviderIds["MusicBrainzArtist"]; ok {
			mediaItem.MusicBrainzArtistID = id
		}
		if id, ok := item.ProviderIds["MusicBrainzAlbum"]; ok {
			mediaItem.MusicBrainzAlbumID = id
		}
		if id, ok := item.ProviderIds["MusicBrainzTrack"]; ok {
			mediaItem.MusicBrainzTrackID = id
		}
		if id, ok := item.ProviderIds["MusicBrainzReleaseGroup"]; ok {
			mediaItem.MusicBrainzReleaseGroupID = id
		}
	}

	return mediaItem
}

func lastSegment(url string) string {
	parts := strings.Split(url, "/")
	if len(parts) > 0 {
		return parts[len(parts)-1]
	}
	return ""
}

type Account struct {
	Name string `json:"Name"`
}

type Request struct {
	Event string  `json:"Event"`
	User  Account `json:"User"`
	Item  Item    `json:"Item"`
}

func HandleRequest(w http.ResponseWriter, r *http.Request) {
	if r.Method != "POST" {
		slog.Warn("MPRIS webhook received unsupported method", "method", r.Method)
		http.Error(w, "Method Not Allowed", http.StatusMethodNotAllowed)
		return
	}

	rq, err := parseRequest(r)
	if err != nil {
		slog.Error("Failed to parse MPRIS request", "error", err)
		http.Error(w, fmt.Sprintf("Bad Request: %s", err.Error()), http.StatusBadRequest)
		return
	}

	common.HandleRequest(rq)
	w.WriteHeader(http.StatusOK)
}

func parseRequest(r *http.Request) (*common.Request, error) {
	var input Request

	bodyBytes, err := io.ReadAll(r.Body)
	if err != nil {
		return nil, fmt.Errorf("failed to read body: %w", err)
	}
	r.Body = io.NopCloser(bytes.NewBuffer(bodyBytes))

	slog.Debug("MPRIS webhook raw body", "body", string(bodyBytes))

	contentType := r.Header.Get("Content-Type")

	switch {
	case strings.Contains(contentType, "application/json"):
		if err := json.NewDecoder(r.Body).Decode(&input); err != nil {
			return nil, fmt.Errorf("failed to decode JSON: %w", err)
		}
	case strings.Contains(contentType, "multipart/form-data"):
		if err := r.ParseMultipartForm(16 << 20); err != nil {
			return nil, fmt.Errorf("failed to parse multipart form: %w", err)
		}
		if err := json.Unmarshal([]byte(r.FormValue("data")), &input); err != nil {
			return nil, fmt.Errorf("failed to unmarshal 'data' field: %w", err)
		}
	default:
		return nil, fmt.Errorf("unsupported Content-Type: %s", contentType)
	}

	if input.Event == "" {
		return nil, errors.New("missing Event")
	}
	if input.Item.Type != "Audio" {
		return nil, fmt.Errorf("unsupported Item.Type: %s", input.Item.Type)
	}
	if input.Item.Track == "" || input.Item.Album == "" {
		return nil, errors.New("missing track or album metadata")
	}
	if input.User.Name == "" {
		return nil, errors.New("missing User.Name")
	}

	rq := &common.Request{
		Event:     input.Event,
		MediaType: strings.ToLower(input.Item.Type),
		User:      input.User.Name,
		Item:      input.Item.AsMediaItem(),
		Source:    "mpris", // ✅ Tower ID goes here
	}

	slog.Debug("Prepared common.Request",
		"user", rq.User,
		"track", rq.Item.Track,
		"album", rq.Item.Album,
		"artist", rq.Item.Artist,
		"mbid_artist", rq.Item.MusicBrainzArtistID,
	)

	return rq, nil
}
